from inductiva.client.paths.tasks_task_id_input.get import ApiForget
from inductiva.client.paths.tasks_task_id_input.put import ApiForput


class TasksTaskIdInput(
        ApiForget,
        ApiForput,
):
    pass
